﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Ink;
using System.Collections.ObjectModel;
using System.Windows.Media.Effects;
using System.Windows.Threading;

namespace WpfApplication1
{
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        // Used for the dragging operations
        private Point startPoint;
        private double originalLeft;
        private double originalTop;
        private bool isDown;
        private bool isDragging;
        private UIElement originalElement;
        private SimpleCircleAdorner overlayElement;

        // For the gestures
        private InkPresenter _inkPresenter = null;
        private Stroke _gestureStroke = null;
        private GestureRecognizer _gestureRecognizer = null;

        // If true then the mouse is being held down and moving.        
        bool moving = false;
        bool highlightsOn = true;
        private UIElement _selectedImage;
        List<Polyline> newLineList = new List<Polyline>();
        Storyboard storyboard = new Storyboard();
        int currentLine = 0;
        int currentBrushIndex = 0;
        List<SolidColorBrush> brushList = new List<SolidColorBrush>();

        // List of objects to iterate through -- turn this to a linked list when we have time
        List<Image> imagesToAnimate = new List<Image>();
        int currentImageIndex = -1;
        bool hiddenPaths = false;

        // Just to try the new class object... 
        List<AnimationObject> animationObjectList = new List<AnimationObject>();
        int currentAnimationObjectIndex = -1;

        public void OnPageLoad(Object sender, RoutedEventArgs e)
        {
            if (_inkPresenter == null)
            {
                _inkPresenter = new InkPresenter();
                canvas.Children.Add(_inkPresenter);
            }
            _gestureRecognizer = new GestureRecognizer();
            canvas.PreviewMouseLeftButtonDown += new System.Windows.Input.MouseButtonEventHandler(canvas_PreviewMouseLeftButtonDown);
            canvas.PreviewMouseMove += new System.Windows.Input.MouseEventHandler(canvas_PreviewMouseMove);
            canvas.PreviewMouseLeftButtonUp += new System.Windows.Input.MouseButtonEventHandler(canvas_PreviewMouseLeftButtonUp);
            this.PreviewKeyDown += new System.Windows.Input.KeyEventHandler(window1_PreviewKeyDown);
            durationSlider.Value = 2;
            
            // Add colored brushes to the brush list (for the paths)
            brushList.Add(new SolidColorBrush(Colors.Yellow));
            brushList.Add(new SolidColorBrush(Colors.Fuchsia));
            brushList.Add(new SolidColorBrush(Colors.Green));
            brushList.Add(new SolidColorBrush(Colors.Orange));
            brushList.Add(new SolidColorBrush(Colors.GreenYellow));
            brushList.Add(new SolidColorBrush(Colors.Turquoise));           
            
        }

        // Goes to the next color in the brush list to change color of the brush
        // Loops to the beginning if it reaches the end
        public void nextColor()
        {
            if (brushList.ElementAtOrDefault(currentBrushIndex + 1) != null)
            {
                currentBrushIndex++;                
            }
            else
            {
                currentBrushIndex = 0;
            }
            displayColor.Background = brushList[currentBrushIndex];

        }

        // Goes to the previous color in the brush list to change color of the brush
        // Loops to the end if it reaches the beginning
        public void prevColor()
        {
            if (brushList.ElementAtOrDefault(currentBrushIndex - 1) != null)
            {
                currentBrushIndex--;
            }
            else
            {
                currentBrushIndex = brushList.Count - 1;
            }
            displayColor.Background = brushList[currentBrushIndex];

        }

        // If user hits the escape key while dragging, cancel the drag
        void window1_PreviewKeyDown(object sender, System.Windows.Input.KeyEventArgs e)
        {
            if (e.Key == System.Windows.Input.Key.Escape && isDragging)
            {
                DragFinished(true);
            }
        }

        // Listen for when the left mouse button is lifted
        void canvas_PreviewMouseLeftButtonUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if (isDown) // If user is currently dragging, end the drag
            {
                DragFinished(false);
                e.Handled = true;
            }
            else
            {
                if (_gestureStroke != null &&  !isDown) // Start gesture recognition
                {
                    Point position = e.GetPosition(this);
                    _gestureStroke.StylusPoints.Add(new StylusPoint(e.GetPosition(canvas).X, e.GetPosition(canvas).Y));
                    restartTimer();
                    StrokeCollection strokesToRecognize = new StrokeCollection();
                    strokesToRecognize.Add(_gestureStroke);
                    ReadOnlyCollection<GestureRecognitionResult> results = _gestureRecognizer.Recognize(strokesToRecognize);
                    GestureRecognitionResult topResult = results[0];


                    if (topResult.RecognitionConfidence == RecognitionConfidence.Strong)
                    {
                        ApplicationGesture gesture = topResult.ApplicationGesture;

                        switch (gesture)
                        {
                         
                            // Delete an animation object
                            case ApplicationGesture.ScratchOut:
                                if (e.Source != canvas)
                                {
                                    canvas.Children.Remove((UIElement)e.Source);

                                }
                                break;
    
                            // Add an image object
                            case ApplicationGesture.DownLeft:
                                Button_Click_6(sender, e);
                                break;

                            // Add frame gesture
                            case ApplicationGesture.UpRight:
                                button1_Click_1(sender, e);
                                break;

                            // Play Storyboard gesture
                            case ApplicationGesture.ChevronRight:
                                storyboard.Begin(this, true);
                                break;

                            case ApplicationGesture.ChevronLeft:
                                storyboard.Stop(this);
                                break;

                            // Tap to select and move an object
                            case ApplicationGesture.Tap:
                                if (e.Source != canvas && !isDragging)
                                {
                                    isDown = true;
                                    startPoint = e.GetPosition(canvas);
                                    originalElement = e.Source as UIElement;
                                    _selectedImage = e.Source as UIElement;

                                    for (int i = 0; i < animationObjectList.Count; i++)
                                    {
                                        int oldAnimationIndex = currentImageIndex;

                                        if (_selectedImage.Equals(animationObjectList[i].image))
                                        {
                                            currentAnimationObjectIndex = i;
                                            currentImageIndex = i;
                                        }

                                        if (imagesToAnimate.Count > 1)
                                        {
                                            removeHighlight(imagesToAnimate[oldAnimationIndex]);
                                        }
                                        highlightImage(imagesToAnimate[currentImageIndex]);
                                        updateInformation();
                                    }

                                    canvas.CaptureMouse();
                                    e.Handled = true;
                                }
                                break;

                            case ApplicationGesture.Curlicue:
                                Console.WriteLine("Curlicue");
                                break;

                        }
                    }
                }

                _inkPresenter.Strokes.Clear();
                _gestureStroke = null;
            }
        }

        // Once dragging is finished, set the new location of the image
        private void DragFinished(bool cancelled)
        {
            System.Windows.Input.Mouse.Capture(null);
            if (isDragging)
            {
                AdornerLayer.GetAdornerLayer(overlayElement.AdornedElement).Remove(overlayElement);

                if (cancelled == false)
                {
                    InkCanvas.SetTop(originalElement, originalTop + overlayElement.TopOffset);
                    InkCanvas.SetLeft(originalElement, originalLeft + overlayElement.LeftOffset);
                }
                overlayElement = null;

            }
            isDragging = false;
            isDown = false;
        }

        // While mouse is moving, update position for dragging
        void canvas_PreviewMouseMove(object sender, System.Windows.Input.MouseEventArgs e)
        {
            if (isDown)
            {
                if ((isDragging == false) && ((Math.Abs(e.GetPosition(canvas).X - startPoint.X) > SystemParameters.MinimumHorizontalDragDistance) ||
                    (Math.Abs(e.GetPosition(canvas).Y - startPoint.Y) > SystemParameters.MinimumVerticalDragDistance)))
                {
                    DragStarted();
                }
                if (isDragging)
                {
                    DragMoved();
                }
            }
            else
            {
                if (_gestureStroke != null)
                {

                    Point position = e.GetPosition(this);
                    _gestureStroke.StylusPoints.Add(new StylusPoint(e.GetPosition(canvas).X, e.GetPosition(canvas).Y));

                }
            }
        }

        // When user starts dragging an image, create the adornder and set status to dragging
        private void DragStarted()
        {
            isDragging = true;
            originalLeft = InkCanvas.GetLeft(originalElement);
            originalTop = InkCanvas.GetTop(originalElement);

            overlayElement = new SimpleCircleAdorner(originalElement);
            AdornerLayer layer = AdornerLayer.GetAdornerLayer(originalElement);
            layer.Add(overlayElement);

        }

        // Update the current position & final position (offset) of where the image is being dragged
        private void DragMoved()
        {
            Point CurrentPosition = System.Windows.Input.Mouse.GetPosition(canvas);

            overlayElement.LeftOffset = CurrentPosition.X - startPoint.X;
            overlayElement.TopOffset = CurrentPosition.Y - startPoint.Y;

        }

        // Left mouse button event listener, 
        void canvas_PreviewMouseLeftButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            
            if (e.Source == canvas && (!isDragging && !isDown))
            {
                // Starts drawing a gesture
                StartTimer();
                if (_inkPresenter == null)
                {
                    _inkPresenter = new InkPresenter();
                    canvas.Children.Add(_inkPresenter);
                }
                Point position = e.GetPosition(canvas);
                _gestureStroke = new Stroke(new StylusPointCollection(new Point[] { position }));
                DrawingAttributes inkAttributes = new DrawingAttributes();
                inkAttributes.Color = Colors.Blue;
                inkAttributes.Height = 5;
                inkAttributes.Width = 5;
                _gestureStroke.DrawingAttributes = inkAttributes;
                _inkPresenter.Strokes.Add(_gestureStroke);

            }
            else // User is starting gesture on an image...
            {
                if (!isDragging && !isDown)
                {
                    // Starts drawing a gesture
                    StartTimer();
                    if (_inkPresenter == null)
                    {
                        _inkPresenter = new InkPresenter();
                        canvas.Children.Add(_inkPresenter);
                    }
                    Point position = e.GetPosition(canvas);
                    _gestureStroke = new Stroke(new StylusPointCollection(new Point[] { position }));
                    DrawingAttributes inkAttributes = new DrawingAttributes();
                    inkAttributes.Color = Colors.Blue;
                    inkAttributes.Height = 5;
                    inkAttributes.Width = 5;
                    _gestureStroke.DrawingAttributes = inkAttributes;
                    _inkPresenter.Strokes.Add(_gestureStroke);
                }
            }
        }

        // Functions to start the timer
        public void StartTimer()
        {
            timeCountInMS = 0;
            DispatcherTimer myDispatcherTimer = new
            DispatcherTimer();
            myDispatcherTimer.Interval = new TimeSpan(0, 0, 0, 0, 1); // 1 Milliseconds 
            myDispatcherTimer.Tick += new EventHandler(Each_Tick);
            myDispatcherTimer.Start();
        }

        // A variable to count the timer
        int timeCountInMS = 0;

        public void Each_Tick(object o, EventArgs sender)
        {
            timeCountInMS++;
        }

        // Restarts the timer
        public void restartTimer()
        {
            timeCountInMS = 0;
        }

        // Returns the laplacian of a list of points
        private List<Point> laplacian(List<Point> animPointList)
        {
            List<Point> animPointListSmoothed = new List<Point>();
            animPointListSmoothed.Add(animPointList.First());
            for (int j = 1; j < animPointList.Count - 1; j++)
            {
                animPointListSmoothed.Add(new Point((animPointList[j - 1].X + animPointList[j + 1].X) / 2, (animPointList[j - 1].Y + animPointList[j + 1].Y) / 2));
            }
            animPointListSmoothed.Add(animPointList.Last());
            return animPointListSmoothed;
        }

        // When the mouse is first right-clicked
        private void Canvas_MouseRightButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (imagesToAnimate.Count > 0)
            {
                // Indicates that the RMB is being held down and moved
                moving = true;

                if (animationObjectList[currentAnimationObjectIndex].initializedPath)
                {

                    int currentLineIndex = animationObjectList[currentAnimationObjectIndex].currentLineIndex;
                    Console.WriteLine("" + currentLineIndex);

                    // If there is already a line for this path then remove it to start drawing another
                    if (animationObjectList[currentAnimationObjectIndex].linePathList.ElementAtOrDefault(currentLineIndex) != null)
                    {
                        canvas.Children.Remove(animationObjectList[currentAnimationObjectIndex].linePathList[currentLineIndex]);
                    }
                    
                }

                // Create a new poly line and register it with a unique name
                Polyline newLine = new Polyline();

                // Set the colour of the brush
                newLine.Stroke = brushList[currentBrushIndex];
                newLine.StrokeThickness = 4;

                String newLineName = "newLine" + newLine.GetHashCode(); // HashCode makes a unique name
                this.RegisterName(newLineName, newLine);

                // Add the new line to the canvas
                canvas.Children.Add(newLine);
                newLine.Points.Add(e.GetPosition(canvas));

                newLineList.Add(newLine);
            }

        }

        // This happens while the right mouse button is being held down
        private void Canvas_MouseMove(object sender, MouseEventArgs e)
        {
            if (moving)
            {
                newLineList[currentLine].Points.Add(e.GetPosition(canvas));
            }

        }

        // This happens when the right mouse button is let go
        private void Canvas_MouseRightButtonUp(object sender, MouseButtonEventArgs e)
        {
            if (moving)
            {
                
                // Now that the mouse button is up, set moving to false indicating that the user is done making the path
                moving = false;

                // Add the last point to the point collection
                newLineList[currentLine].Points.Add(e.GetPosition(canvas));
                currentLine++;
                Image imageToAnimate = imagesToAnimate[currentImageIndex];

                // Create point collection using the points from the polyline the user created
                PointCollection animPointCollection = newLineList[currentLine - 1].Points;
                
                // Create the path geometry using the point collection
                PathGeometry pg = new PathGeometry();
                pg.FillRule = FillRule.Nonzero;
                PathFigure pf = new PathFigure();

                // Add each point to a path figure to add to the path geometry
                int counter = 0;

                // These offsets are used to make the animation use the center of the image
                double xOffset = imageToAnimate.ActualWidth / -2;
                double yOffset = imageToAnimate.ActualHeight / -2;

                // Get a list out of the point collection that the user drew
                List<Point> animPointList = animPointCollection.ToList();

                // lolplacian
                List<Point> animPointListSmoothed = laplacian(laplacian(laplacian(laplacian(laplacian(animPointList))))); 
           
                // replace the points in the line with the new bi-laplacian
                for (int k = 0; k < animPointListSmoothed.Count; k++)
                {
                    newLineList[currentLine - 1].Points[k] = animPointListSmoothed[k];
                }

                foreach (Point p in animPointCollection)
                {
                    if (counter == 0)  // Get start point
                    {                        
                        pf.StartPoint = Point.Add(p, new Vector(xOffset, yOffset));
                    }
                    pf.Segments.Add(new LineSegment(Point.Add(p, new Vector(xOffset, yOffset)), true));
                    counter++;
                }
                
                pg.Figures.Add(pf);
                pg.Freeze(); // Improves performance
                
                if (animationObjectList[currentAnimationObjectIndex].initializedPath == false)
                {
                    // First path for the object
                    addTranslationPath(animationObjectList[currentAnimationObjectIndex], pg, TimeSpan.FromSeconds(0));
 
                } else {
                    
                    int currentLineIndex = animationObjectList[currentAnimationObjectIndex].currentLineIndex;

                    if (animationObjectList[currentAnimationObjectIndex].daupXList.Count > 0)
                    {
                        addTranslationPath(animationObjectList[currentAnimationObjectIndex], pg, TimeSpan.FromSeconds(0));
                        updateTiming(animationObjectList[currentAnimationObjectIndex]);
                    }
                    else
                    {
                        // Then it's the first animation so start it at 0
                        addTranslationPath(animationObjectList[currentAnimationObjectIndex], pg, TimeSpan.FromSeconds(0));
                    }
                }
            }
        }

        // Adds a translation path to the animation object
        private void addTranslationPath(AnimationObject animObject, PathGeometry pg, Duration startTime)
        {
            
            Image imageToAnimate = animObject.image;
            
            // Transform Group  
            TransformGroup animeTransformGroup = new TransformGroup();

            // Create some transforms. These transforms
            // will be used to move and rotate the rectangle.
            RotateTransform rt = new RotateTransform();
            TranslateTransform tt = new TranslateTransform();

            String rtName = "rt" + rt.GetHashCode();
            String ttName = "tt" + tt.GetHashCode();
            this.RegisterName(ttName, tt);
            this.RegisterName(rtName, rt);

            animeTransformGroup.Children.Add(tt);
            animeTransformGroup.Children.Add(rt);
            imageToAnimate.RenderTransform = animeTransformGroup;

            DoubleAnimation rotAnimation = new DoubleAnimation();
            rotAnimation.From = 0;
            rotAnimation.To = rotAmountSlider.Value;
            rotAnimation.Duration = new Duration(TimeSpan.FromSeconds(rotDurationSlider.Value));

            if (rotRepeatCheckBox.IsChecked == true)
            {
                rotAnimation.RepeatBehavior = RepeatBehavior.Forever;
            }
            else
            {
                rotAnimation.RepeatBehavior = new RepeatBehavior(1);
            }

            if (rotAutoRevCheckBox.IsChecked == true)
            {
                rotAnimation.AutoReverse = true;
            }
            else
            {
                rotAnimation.AutoReverse = false;
            }
            imageToAnimate.RenderTransformOrigin = new Point(0.5, 0.5);

            // Create the animation using the path geometry
            DoubleAnimationUsingPath daupX = new DoubleAnimationUsingPath();
            DoubleAnimationUsingPath daupY = new DoubleAnimationUsingPath();
            daupX.PathGeometry = new PathGeometry();
            daupX.PathGeometry.AddGeometry(pg);
            daupY.PathGeometry = new PathGeometry();
            daupY.PathGeometry.AddGeometry(pg);

            // To do: let user define the time span in some way.
            daupX.BeginTime = startTime.TimeSpan;
            daupX.Duration = TimeSpan.FromSeconds(double.Parse(durationSlider.Value.ToString()));
            daupY.BeginTime = startTime.TimeSpan;
            daupY.Duration = TimeSpan.FromSeconds(double.Parse(durationSlider.Value.ToString()));


            // Set the Source property to X or Y. This makes
            // the animation generate horizontal/vertical offset values from
            // the path information.
            daupX.Source = PathAnimationSource.X;
            daupY.Source = PathAnimationSource.Y;

            // This makes the animation stop at the end of its progress but it breaks drag 'n drop need to toggle
            daupX.FillBehavior = FillBehavior.HoldEnd;
            daupY.FillBehavior = FillBehavior.HoldEnd;
            rotAnimation.FillBehavior = FillBehavior.HoldEnd;

            // Set the animation to target the Angle property
            // of the RotateTransform named "AnimatedRotateTransform".
            Storyboard.SetTargetName(rotAnimation, rtName);
            Storyboard.SetTargetProperty(rotAnimation, new PropertyPath(RotateTransform.AngleProperty));

            // Sets the X animation on the object and makes it move
            // relative to the Left side (horizontal) of the canvas
            Storyboard.SetTarget(daupX, imageToAnimate);
            Storyboard.SetTargetProperty(daupX, new PropertyPath("(InkCanvas.Left)"));

            // Sets the Y animation on the object and makes it move
            // relative to the Top side (vertical) of the canvas
            Storyboard.SetTarget(daupY, imageToAnimate);
            Storyboard.SetTargetProperty(daupY, new PropertyPath("(InkCanvas.Top)"));

            // Add the animations to the storyboard
            storyboard.Children.Add(rotAnimation);
            storyboard.Children.Add(daupX);
            storyboard.Children.Add(daupY);

            // Add the info to the animation object
            if (animationObjectList[currentAnimationObjectIndex].initializedPath == false)
            {
                animationObjectList[currentAnimationObjectIndex].linePathList = new List<Polyline>();
            }
            
            if (animationObjectList[currentAnimationObjectIndex].initializedPath == false)
            {
                animationObjectList[currentAnimationObjectIndex].daupXList = new List<DoubleAnimationUsingPath>();
                animationObjectList[currentAnimationObjectIndex].daupYList = new List<DoubleAnimationUsingPath>();
            }
            
            int currentLineIndex = animationObjectList[currentAnimationObjectIndex].currentLineIndex;

            // If there is a line at this place already, then replace it.
            if (animationObjectList[currentAnimationObjectIndex].linePathList.ElementAtOrDefault(currentLineIndex) != null)
            {
                animationObjectList[currentAnimationObjectIndex].linePathList[currentLineIndex] = newLineList[currentLine - 1];
            }
            else
            {
                animationObjectList[currentAnimationObjectIndex].linePathList.Add(newLineList[currentLine - 1]);
            }

            if (animationObjectList[currentAnimationObjectIndex].daupYList.ElementAtOrDefault(currentLineIndex) != null)
            {
                animationObjectList[currentAnimationObjectIndex].daupXList[currentLineIndex] = daupX;
                animationObjectList[currentAnimationObjectIndex].daupYList[currentLineIndex] = daupY;
            }
            else
            {
                animationObjectList[currentAnimationObjectIndex].daupXList.Add(daupX);
                animationObjectList[currentAnimationObjectIndex].daupYList.Add(daupY);
            }
            animationObjectList[currentAnimationObjectIndex].rotationAnimation = rotAnimation;

            // Preserve settings
            if (animationObjectList[currentAnimationObjectIndex].repeat)
            {
                animationObjectList[currentAnimationObjectIndex].daupXList[currentLineIndex].RepeatBehavior = RepeatBehavior.Forever;
                animationObjectList[currentAnimationObjectIndex].daupYList[currentLineIndex].RepeatBehavior = RepeatBehavior.Forever;
            }

            if (animationObjectList[currentAnimationObjectIndex].autoReverse)
            {
                animationObjectList[currentAnimationObjectIndex].daupXList[currentLineIndex].AutoReverse = true;
                animationObjectList[currentAnimationObjectIndex].daupYList[currentLineIndex].AutoReverse = true;
            }

            animationObjectList[currentAnimationObjectIndex].initializedPath = true;

        }

        // PLAY
        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            storyboard.Begin(this, true);
        }

        // Stop button
        private void Button_Click_7(object sender, RoutedEventArgs e)
        {
            storyboard.Stop(this);
        }
        public bool paused = false;

        // Pause / Resume button
        private void Button_Click_8(object sender, RoutedEventArgs e)
        {
            if (paused == false)
            {
                storyboard.Pause(this);
                paused = true;
            }
            else
            {
                storyboard.Resume(this);
                paused = false;
            }
        }

        // Add image object
        private void Button_Click_6(object sender, RoutedEventArgs e)
        {
            Microsoft.Win32.OpenFileDialog dlg = new Microsoft.Win32.OpenFileDialog();
            dlg.FileName = ""; // Default file name -- could delete this line-
            dlg.Filter = "Image files (*.png, *.jpg, *.jpeg *.bmp)|*.png;*.jpg;*.jpeg;*.bmp|PNG files (*.png)|*.png|JPG files (*.jpg)|*.jpg|JPEG files (*.jpeg)|*.jpg|Bitmap files (*.bmp)|*.bmp"; // Filter files by extension

            if (dlg.ShowDialog() == true)
            {
                Image image = new Image
                {
                    Source = new BitmapImage(new Uri(dlg.FileName))
                };
                canvas.Children.Add(image);
                InkCanvas.SetTop(image, 300);
                InkCanvas.SetLeft(image, 300);
                imagesToAnimate.Add(image);
                currentImageIndex = imagesToAnimate.Count - 1;
                currentAnimationObjectIndex = imagesToAnimate.Count - 1;
                highlightImage(imagesToAnimate[currentImageIndex]);

                if (imagesToAnimate.Count > 1)
                {
                    removeHighlight(imagesToAnimate[currentAnimationObjectIndex - 1]);
                }
                // Create an animation object
                AnimationObject newAnimObject = new AnimationObject();
                newAnimObject.image = image;
                animationObjectList.Add(newAnimObject);

                // Transform Group  
                TransformGroup animeTransformGroup = new TransformGroup();

                // Create some transforms. These transforms
                // will be used to move and rotate the rectangle.
                RotateTransform rt = new RotateTransform();

                String rtName = "rt" + rt.GetHashCode();
                this.RegisterName(rtName, rt);

                image.RenderTransform = animeTransformGroup;

                animeTransformGroup.Children.Add(rt);

                DoubleAnimation rotAnimation = new DoubleAnimation();
                rotAnimation.From = 0;
                rotAnimation.To = rotAmountSlider.Value;
                rotAnimation.Duration = new Duration(TimeSpan.FromSeconds(rotDurationSlider.Value));

                if (rotRepeatCheckBox.IsChecked == true)
                {
                    rotAnimation.RepeatBehavior = RepeatBehavior.Forever;
                }
                else
                {
                    rotAnimation.RepeatBehavior = new RepeatBehavior(1);
                }
                
                if (rotAutoRevCheckBox.IsChecked == true)
                {
                    rotAnimation.AutoReverse = true;
                }
                else
                {
                    rotAnimation.AutoReverse = false;
                }

                image.RenderTransformOrigin = new Point(0.5, 0.5);
                rotAnimation.FillBehavior = FillBehavior.HoldEnd;

                // Set the animation to target the Angle property
                // of the RotateTransform named "AnimatedRotateTransform".
                Storyboard.SetTargetName(rotAnimation, rtName);
                Storyboard.SetTargetProperty(rotAnimation, new PropertyPath(RotateTransform.AngleProperty));
                
                storyboard.Children.Add(rotAnimation);

                animationObjectList[currentAnimationObjectIndex].rotationAnimation = rotAnimation;
                animationObjectList[currentAnimationObjectIndex].currentLineIndex = 0;
                animationObjectList[currentAnimationObjectIndex].initialized = true;
                bounceCheckbox.SetCurrentValue(CheckBox.IsCheckedProperty, false);
                repeatCheckbox.SetCurrentValue(CheckBox.IsCheckedProperty, false);

                animationObjectList[currentAnimationObjectIndex].frameDuration += double.Parse(textBox1.Text);

            }
        }

        // Add frame
        private void button1_Click_1(object sender, RoutedEventArgs e)
        {
            Microsoft.Win32.OpenFileDialog dlg = new Microsoft.Win32.OpenFileDialog();
            dlg.FileName = ""; // Default file name -- could delete this line
            dlg.Filter = "Image files (*.png, *.jpg, *.jpeg *.bmp)|*.png;*.jpg;*.jpeg;*.bmp|PNG files (*.png)|*.png|JPG files (*.jpg)|*.jpg|JPEG files (*.jpeg)|*.jpg|Bitmap files (*.bmp)|*.bmp"; // Filter files by extension

            if (dlg.ShowDialog() == true)
            {
                Image image = new Image
                {
                    Source = new BitmapImage(new Uri(dlg.FileName))
                };

                if (animationObjectList[currentAnimationObjectIndex].myKeyFrames == null)
                {
                    Image temp = new Image();
                    temp.Source = animationObjectList[currentAnimationObjectIndex].image.Source;
                    animationObjectList[currentAnimationObjectIndex].frames = new List<Image>();
                    animationObjectList[currentAnimationObjectIndex].frames.Add(temp);

                    animationObjectList[currentAnimationObjectIndex].myKeyFrames = new ObjectAnimationUsingKeyFrames();
                    animationObjectList[currentAnimationObjectIndex].myKeyFrames.AutoReverse = 
                    	animationObjectList[currentAnimationObjectIndex].frameReverse;
                    if (animationObjectList[currentAnimationObjectIndex].frameRepeat == true){
                    	animationObjectList[currentAnimationObjectIndex].myKeyFrames.RepeatBehavior = RepeatBehavior.Forever;
                    }
                    else{
                    	animationObjectList[currentAnimationObjectIndex].myKeyFrames.RepeatBehavior = new RepeatBehavior(1);
                    }

                    animationObjectList[currentAnimationObjectIndex].myKeyFrames.KeyFrames.Add(
                        new DiscreteObjectKeyFrame(temp.Source, KeyTime.FromTimeSpan(TimeSpan.FromSeconds(0))));

                    storyboard.Children.Add(animationObjectList[currentAnimationObjectIndex].myKeyFrames);
                    Storyboard.SetTarget(animationObjectList[currentAnimationObjectIndex].myKeyFrames,
                          animationObjectList[currentAnimationObjectIndex].image);
                    Storyboard.SetTargetProperty(animationObjectList[currentAnimationObjectIndex].myKeyFrames,
                        new PropertyPath(Image.SourceProperty));
                }

                animationObjectList[currentAnimationObjectIndex].frames.Add(image);

                
                animationObjectList[currentAnimationObjectIndex].myKeyFrames.KeyFrames.Add(
                    new DiscreteObjectKeyFrame(image.Source, KeyTime.FromTimeSpan(
                		TimeSpan.FromSeconds(animationObjectList[currentAnimationObjectIndex].frameDuration))));
                animationObjectList[currentAnimationObjectIndex].frameDuration += double.Parse(textBox1.Text);
                animationObjectList[currentAnimationObjectIndex].myKeyFrames.Duration = 
                	TimeSpan.FromSeconds(animationObjectList[currentAnimationObjectIndex].frameDuration);
            }
        }

        // Repeat checkbox checked
        private void checkBox1_Checked(object sender, RoutedEventArgs e)
        {
            int currentLineIndex = animationObjectList[currentAnimationObjectIndex].currentLineIndex;

            if (animationObjectList.Count > 0 && animationObjectList[currentAnimationObjectIndex].initializedPath)
            {
                animationObjectList[currentAnimationObjectIndex].daupXList[currentLineIndex].RepeatBehavior = RepeatBehavior.Forever;
                animationObjectList[currentAnimationObjectIndex].daupYList[currentLineIndex].RepeatBehavior = RepeatBehavior.Forever;
                animationObjectList[currentAnimationObjectIndex].repeat = true;
            }

        }

        // Repeat checkbox unchecked
        private void Repeat_Unchecked(object sender, RoutedEventArgs e)
        {
            int currentLineIndex = animationObjectList[currentAnimationObjectIndex].currentLineIndex;

            if (animationObjectList.Count > 0 && animationObjectList[currentAnimationObjectIndex].initializedPath)
            {
                animationObjectList[currentAnimationObjectIndex].daupXList[currentLineIndex].RepeatBehavior = new RepeatBehavior(1);
                animationObjectList[currentAnimationObjectIndex].daupYList[currentLineIndex].RepeatBehavior = new RepeatBehavior(1);
                animationObjectList[currentAnimationObjectIndex].repeat = false;
            }
        }

        // Bounce checkbox checked
        private void checkBox2_Checked(object sender, RoutedEventArgs e)
        {
            int currentLineIndex = animationObjectList[currentAnimationObjectIndex].currentLineIndex;

            if (animationObjectList.Count > 0 && animationObjectList[currentAnimationObjectIndex].initializedPath)
            {
                animationObjectList[currentAnimationObjectIndex].daupXList[currentLineIndex].AutoReverse = true;
                animationObjectList[currentAnimationObjectIndex].daupYList[currentLineIndex].AutoReverse = true;
                animationObjectList[currentAnimationObjectIndex].autoReverse = true;
            }
        }

        // Bounce checkbox unchecked
        private void Bounce_Unchecked(object sender, RoutedEventArgs e)
        {

            int currentLineIndex = animationObjectList[currentAnimationObjectIndex].currentLineIndex;

            if (animationObjectList.Count > 0 && animationObjectList[currentAnimationObjectIndex].initializedPath)
            {
                animationObjectList[currentAnimationObjectIndex].daupXList[currentLineIndex].AutoReverse = false;
                animationObjectList[currentAnimationObjectIndex].daupYList[currentLineIndex].AutoReverse = false;
                animationObjectList[currentAnimationObjectIndex].autoReverse = false;
            }
        }

        /*
         * Takes an image and adds a highlight around it it.
         * Currently used for showing that an image is selected.
         * @param imageToHighlight The image that will be highlighted.
         */
        private void highlightImage(Image imageToHighlight)
        {
            if (highlightsOn)
            {
                DropShadowEffect shadowForImage = new DropShadowEffect();
                shadowForImage.BlurRadius = 20;
                shadowForImage.ShadowDepth = 0;
                shadowForImage.Opacity = .75;
                shadowForImage.Color = Colors.Aqua;
                imageToHighlight.Effect = shadowForImage;
            }
        }

        /*
        * Removes a highlight from an image.        
        * @param imageToHighlight The image that will have its highlight removed.
        */
        private void removeHighlight(Image imageToHighlight)
        {
            // Note: There is probably a better way to get rid of the effect...            
            DropShadowEffect shadowForImage = new DropShadowEffect();
            shadowForImage.BlurRadius = 0;
            shadowForImage.ShadowDepth = 0;
            shadowForImage.Opacity = 0;
            imageToHighlight.Effect = shadowForImage;
        }


        /* 
         * Updates the information window to show the correct information
         * like the bounce/repeat for the selected object.
         */
        private void updateInformation()
        {
            if (animationObjectList[currentAnimationObjectIndex].autoReverse)
            {
                bounceCheckbox.SetCurrentValue(CheckBox.IsCheckedProperty, true);
            }
            else
            {
                bounceCheckbox.SetCurrentValue(CheckBox.IsCheckedProperty, false);
            }

            if (animationObjectList[currentAnimationObjectIndex].repeat)
            {
                repeatCheckbox.SetCurrentValue(CheckBox.IsCheckedProperty, true);
            }
            else
            {
                repeatCheckbox.SetCurrentValue(CheckBox.IsCheckedProperty, false);
            }

            if (animationObjectList[currentAnimationObjectIndex].rotRepeat)
            {
                rotRepeatCheckBox.SetCurrentValue(CheckBox.IsCheckedProperty, true);
            }
            else
            {
                rotRepeatCheckBox.SetCurrentValue(CheckBox.IsCheckedProperty, false);
            }

            if (animationObjectList[currentAnimationObjectIndex].rotAutoReverse)
            {
                rotAutoRevCheckBox.SetCurrentValue(CheckBox.IsCheckedProperty, true);
            }
            else
            {
                rotAutoRevCheckBox.SetCurrentValue(CheckBox.IsCheckedProperty, false);
            }
            
            if (animationObjectList[currentAnimationObjectIndex].frameRepeat)
            {
                frameRepeat.SetCurrentValue(CheckBox.IsCheckedProperty, true);
            }
            else
            {
                frameRepeat.SetCurrentValue(CheckBox.IsCheckedProperty, false);
            }

            if (animationObjectList[currentAnimationObjectIndex].frameReverse)
            {
                frameReverse.SetCurrentValue(CheckBox.IsCheckedProperty, true);
            }
            else
            {
                frameReverse.SetCurrentValue(CheckBox.IsCheckedProperty, false);
            }

            rotDurationSlider.Value = animationObjectList[currentAnimationObjectIndex].rotDuration;
            rotAmountSlider.Value = animationObjectList[currentAnimationObjectIndex].rotAmount;
        }

        // Next object button -- we can remove this when click-selection is working
        private void button1_Click(object sender, RoutedEventArgs e)
        {
            if (currentImageIndex < imagesToAnimate.Count - 1)
            {
                removeHighlight(imagesToAnimate[currentImageIndex]);
                currentImageIndex++;
                currentAnimationObjectIndex++;
                highlightImage(imagesToAnimate[currentImageIndex]);
            }
            updateInformation();
        }


        // Previous object button -- we can remove this when click-selection is working
        private void button2_Click(object sender, RoutedEventArgs e)
        {
            if (currentImageIndex > 0)
            {
                removeHighlight(imagesToAnimate[currentImageIndex]);
                currentImageIndex--;
                currentAnimationObjectIndex--;
                highlightImage(imagesToAnimate[currentImageIndex]);
            }
            updateInformation();
        }

        // Hide polylines button
        private void button3_Click(object sender, RoutedEventArgs e)
        {
            if (hiddenPaths == false)
            {
                for (int i = 0; i < newLineList.Count; i++)
                {
                    newLineList[i].StrokeThickness = 0;
                }
                hiddenPaths = true;
            }
            else
            {
                for (int i = 0; i < newLineList.Count; i++)
                {
                    newLineList[i].StrokeThickness = 4;
                }
                hiddenPaths = false;
            }

        }

        // Select background button
        private void button4_Click(object sender, RoutedEventArgs e)
        {
            Microsoft.Win32.OpenFileDialog dlg = new Microsoft.Win32.OpenFileDialog();
            dlg.FileName = ""; // Default file name -- could delete this line
            dlg.Filter = "Image files (*.png, *.jpg, *.jpeg *.bmp)|*.png;*.jpg;*.jpeg;*.bmp|PNG files (*.png)|*.png|JPG files (*.jpg)|*.jpg|JPEG files (*.jpeg)|*.jpg|Bitmap files (*.bmp)|*.bmp"; // Filter files by extension

            if (dlg.ShowDialog() == true)
            {
                ImageBrush imageBrush = new ImageBrush
                {
                    ImageSource = new BitmapImage(new Uri(dlg.FileName))
                };

                canvas.Background = imageBrush;

            }
        }

        // Hide highlight button
        private void button5_Click(object sender, RoutedEventArgs e)
        {
            if (highlightsOn)
            {
                removeHighlight(animationObjectList[currentAnimationObjectIndex].image);
                highlightButton.Content = "Show highlight";
                highlightsOn = false;
            }
            else
            {
                highlightsOn = true;
                highlightImage(animationObjectList[currentAnimationObjectIndex].image);
                highlightButton.Content = "Hide highlight";
            }

        }

        // Listen for if rotation auto-reverse checkbox is checked
        private void rotAutoRev_Checked(object sender, RoutedEventArgs e)
        {
            if (animationObjectList.Count > 0 && animationObjectList[currentAnimationObjectIndex].initialized)
            {
                animationObjectList[currentAnimationObjectIndex].rotationAnimation.AutoReverse = true;
                animationObjectList[currentAnimationObjectIndex].rotAutoReverse = true;
            }
        }

        // Listen for if rotation auto-reverse checkbox is unchecked
        private void rotAutoRev_Unchecked(object sender, RoutedEventArgs e)
        {
            if (animationObjectList.Count > 0 && animationObjectList[currentAnimationObjectIndex].initialized)
            {
                animationObjectList[currentAnimationObjectIndex].rotationAnimation.AutoReverse = false;
                animationObjectList[currentAnimationObjectIndex].rotAutoReverse = false;
            }
        }

        // Listen for if rotation repeat checkbox is checked
        private void rotRepeat_Checked(object sender, RoutedEventArgs e)
        {
            if (animationObjectList.Count > 0 && animationObjectList[currentAnimationObjectIndex].initialized)
            {
                animationObjectList[currentAnimationObjectIndex].rotationAnimation.RepeatBehavior = RepeatBehavior.Forever;
                animationObjectList[currentAnimationObjectIndex].rotRepeat = true;
            }
        }

        // Listen for if rotation repeat checkbox is unchecked
        private void rotRepeat_Unchecked(object sender, RoutedEventArgs e)
        {
            if (animationObjectList.Count > 0 && animationObjectList[currentAnimationObjectIndex].initialized)
            {
                animationObjectList[currentAnimationObjectIndex].rotationAnimation.RepeatBehavior = new RepeatBehavior(1);
                animationObjectList[currentAnimationObjectIndex].rotRepeat = false;
            }
        }

        // Update the duration of rotation animations if slider value changed
        private void rotDurationSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (currentAnimationObjectIndex > -1 && animationObjectList[currentAnimationObjectIndex].initialized)
            {
                animationObjectList[currentAnimationObjectIndex].rotationAnimation.Duration = new Duration(TimeSpan.FromSeconds(rotDurationSlider.Value));
                animationObjectList[currentAnimationObjectIndex].rotDuration = rotDurationSlider.Value;

                rotDurationTextBox.Text = rotDurationSlider.Value.ToString();
            }
        }

        // Update the amount of rotation animations if slider value changed
        private void rotAmountSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (currentAnimationObjectIndex > -1 && animationObjectList[currentAnimationObjectIndex].initialized)
            {
                animationObjectList[currentAnimationObjectIndex].rotationAnimation.To = rotAmountSlider.Value;
                animationObjectList[currentAnimationObjectIndex].rotAmount = rotAmountSlider.Value;                
            }

            rotAmountTextBox.Text = rotAmountSlider.Value.ToString();
        }

        // Update the duration of translation animations if slider value changed
        private void durationSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (currentAnimationObjectIndex > -1 && animationObjectList[currentAnimationObjectIndex].initialized)
            {
                int currentLineIndex = animationObjectList[currentAnimationObjectIndex].currentLineIndex;


                //.ElementAtOrDefault(currentAnimationObjectIndex) 

                if (animationObjectList[currentAnimationObjectIndex].daupXList != null)
                {
                    Console.WriteLine("update duration");
                    animationObjectList[currentAnimationObjectIndex].daupXList[currentLineIndex].Duration = TimeSpan.FromSeconds(double.Parse(durationSlider.Value.ToString()));
                    animationObjectList[currentAnimationObjectIndex].daupYList[currentLineIndex].Duration = TimeSpan.FromSeconds(double.Parse(durationSlider.Value.ToString()));
                }
                                
                animationObjectList[currentAnimationObjectIndex].transDuration = double.Parse(durationSlider.Value.ToString());
            }

            durationTextBox.Text = durationSlider.Value.ToString();

            // This may cause problems with the "stuttering"
            if (animationObjectList.ElementAtOrDefault(0) != null && animationObjectList[currentAnimationObjectIndex].daupXList != null)
            {
                Console.WriteLine("update duration 2");
                updateTiming(animationObjectList[currentAnimationObjectIndex]);
            }
        }
    
        // Update timing if a path time is changed
        private void updateTiming(AnimationObject animObject)
        {
            Console.WriteLine("Updating the timing");
            
            // Remove the existing animations from the storyboard for this object.
            // This increases the effeciency, reducing the stuttering problems
            storyboard.Children.Remove(animObject.daupXList[0]);
            storyboard.Children.Remove(animObject.daupYList[0]);
            
            storyboard.Children.Add(animObject.daupXList[0]);
            storyboard.Children.Add(animObject.daupYList[0]);
            double totalTime = 0;
            for (int i = 1; i < animObject.daupXList.Count; i++)
            {
                storyboard.Children.Remove(animObject.daupXList[i]); // Remove existing animations
                storyboard.Children.Remove(animObject.daupYList[i]);

                totalTime += animObject.daupXList[i - 1].Duration.TimeSpan.TotalSeconds;
                animObject.daupXList[i].BeginTime = TimeSpan.FromSeconds(totalTime);
                animObject.daupYList[i].BeginTime = TimeSpan.FromSeconds(totalTime);

                storyboard.Children.Add(animObject.daupXList[i]); // Replace with updated ones
                storyboard.Children.Add(animObject.daupYList[i]);

            }
        }

        // Selects the next path for the animation object
        // Returns true if there is a path and it is selected
        private bool nextPath(AnimationObject animObject)
        {
            if (animObject.linePathList != null)
            {
                Console.WriteLine("nextPath true first current line index: " + animObject.currentLineIndex);
                if (animObject.linePathList.ElementAtOrDefault(animObject.currentLineIndex + 1) != null)
                {
                    animObject.currentLineIndex++;
                    nextColor();
                    Console.WriteLine("nextPath true current line index: " + animObject.currentLineIndex);
                    return true;
                }
            }
            Console.WriteLine("nextPath false current line index: " + animObject.currentLineIndex);
            return false;
        }

        // Selects the previous path for the animation object
        // Returns true if there is a path and it is selected
        private bool prevPath(AnimationObject animObject)
        {
            if (animObject.linePathList != null)
            {
                if (animObject.currentLineIndex == 0)
                {
                    Console.WriteLine("PrevPath false current line index: " + animObject.currentLineIndex); ;
                    return false;
                }
                animObject.currentLineIndex--;
                prevColor();
                Console.WriteLine("PrevPath true current line index: " + animObject.currentLineIndex);
                return true;
            }
            Console.WriteLine("PrevPath false current line index: " + animObject.currentLineIndex);
            return false;
        }

        // Adds a next path for the animation object
        // Returns true if there is a path and it is selected
        private bool appendPath(AnimationObject animObject)
        {
            if (animObject.linePathList != null)
            {
                
                animObject.currentLineIndex++;
                nextColor();
                Console.WriteLine("AppendPath true -- current line index: " + animObject.currentLineIndex);
                return true;
            }
            Console.WriteLine("AppendPath false current line index: " + animObject.currentLineIndex);
            return false;
        }

        // Changes the color and the current translation animation for an object, allows for different timing for one object
        private void canvas_MouseWheel(object sender, MouseWheelEventArgs e)
        {

            if (currentAnimationObjectIndex != -1)
            {
                //Check direction of mouse wheel            
                if (e.Delta > 0)
                {
                    prevPath(animationObjectList[currentAnimationObjectIndex]);
                }
                else
                {
                    if (animationObjectList[currentAnimationObjectIndex].daupXList.ElementAtOrDefault(animationObjectList[currentAnimationObjectIndex].currentLineIndex + 1) != null)
                    {
                        nextPath(animationObjectList[currentAnimationObjectIndex]);
                    }
                    else
                    {
                        appendPath(animationObjectList[currentAnimationObjectIndex]);
                    }
                }
            }
        }
        
        // If the reverse frames checkbox is checked, update keyframe property
		public void FrameReverse_Checked(object sender, RoutedEventArgs e)
		{
			animationObjectList[currentAnimationObjectIndex].frameReverse = true;
			if (animationObjectList[currentAnimationObjectIndex].myKeyFrames!= null){
				animationObjectList[currentAnimationObjectIndex].myKeyFrames.AutoReverse = true;
			}
		}

        //  If the reverse frames checkbox is unchecked, update keyframe property
		public void FrameReverse_Unchecked(object sender, RoutedEventArgs e)
		{
			animationObjectList[currentAnimationObjectIndex].frameReverse = false;
			if (animationObjectList[currentAnimationObjectIndex].myKeyFrames!= null){
				animationObjectList[currentAnimationObjectIndex].myKeyFrames.AutoReverse = false;
			}
		}

        // If the repear frames checkbox is checked, update keyframe property
		public void FrameRepeat_Checked(object sender, RoutedEventArgs e)
		{
			animationObjectList[currentAnimationObjectIndex].frameRepeat = true;
			if (animationObjectList[currentAnimationObjectIndex].myKeyFrames!= null){
				animationObjectList[currentAnimationObjectIndex].myKeyFrames.RepeatBehavior = RepeatBehavior.Forever;
			}
		}

        // If the reverse frames checkbox is unchecked, update keyframe property
		public void FrameRepeat_Unchecked(object sender, RoutedEventArgs e)
		{
			animationObjectList[currentAnimationObjectIndex].frameRepeat = false;
			if (animationObjectList[currentAnimationObjectIndex].myKeyFrames!= null){
				animationObjectList[currentAnimationObjectIndex].myKeyFrames.RepeatBehavior = new RepeatBehavior(1);
			}
		}
		
    }


    /**
     * This is the object that holds user defined propertes for animation objects.
     */
    public class AnimationObject
    {
        
        public Image image;
        public bool initialized = false;
        
        // Properties for the translation animation
        public List<DoubleAnimationUsingPath> daupXList;
        public List<DoubleAnimationUsingPath> daupYList;
        public List<Polyline> linePathList;
        public int currentLineIndex = 0;
        public bool repeat = false;
        public bool autoReverse = false;
        public bool initializedPath = false;
        public double transDuration;

        // The offset to the path that the object will follow (center of image)
        public double xOffset;
        public double yOffset;

        // Properties for rotation animation
        public DoubleAnimation rotationAnimation;        
        public bool rotAutoReverse = false;
        public double rotDuration;
        public double rotAmount;
        public bool rotRepeat = false;

        // For frame-by-frame style animation
        public List<Image> frames;
        public ObjectAnimationUsingKeyFrames myKeyFrames;
        public bool frameReverse = false;
        public bool frameRepeat = false;
        public double frameDuration = 0;
    }

    // Adorner for the dragging of an image
    // Used sample from MSDN for the basis of drag and drop
    // msdn.microsoft.com/en-us/library/bb295243(v=vs.90).aspx
    public class SimpleCircleAdorner : Adorner
    {
        // Be sure to call the base class constructor.
        public SimpleCircleAdorner(UIElement adornedElement)
            : base(adornedElement)
        {
            VisualBrush _brush = new VisualBrush(adornedElement);

            _child = new Rectangle();
            _child.Width = adornedElement.RenderSize.Width * 1.1;
            _child.Height = adornedElement.RenderSize.Height * 1.1;

            DoubleAnimation animation = new DoubleAnimation(0.3, 1, new Duration(TimeSpan.FromSeconds(1)));
            animation.AutoReverse = true;
            animation.RepeatBehavior = System.Windows.Media.Animation.RepeatBehavior.Forever;
            _brush.BeginAnimation(System.Windows.Media.Brush.OpacityProperty, animation);

            _child.Fill = _brush;
        }

        // A common way to implement an adorner's rendering behavior is to override the OnRender
        // method, which is called by the layout subsystem as part of a rendering pass.
        protected override void OnRender(DrawingContext drawingContext)
        {
            // Get a rectangle that represents the desired size of the rendered element
            // after the rendering pass.  This will be used to draw at the corners of the 
            // adorned element.
            Rect adornedElementRect = new Rect(this.AdornedElement.DesiredSize);

            // Some arbitrary drawing implements.
            SolidColorBrush renderBrush = new SolidColorBrush(Colors.Green);
            renderBrush.Opacity = 0;
            Pen renderPen = new Pen(new SolidColorBrush(Colors.Navy), 1.5);

            // Just draw a circle at each corner.
            drawingContext.DrawRectangle(renderBrush, renderPen, adornedElementRect);
        }

        protected override Size MeasureOverride(Size constraint)
        {
            _child.Measure(constraint);
            return _child.DesiredSize;
        }

        protected override Size ArrangeOverride(Size finalSize)
        {
            _child.Arrange(new Rect(finalSize));
            return finalSize;
        }

        protected override Visual GetVisualChild(int index)
        {
            return _child;
        }

        protected override int VisualChildrenCount
        {
            get
            {
                return 1;
            }
        }

        public double LeftOffset
        {
            get
            {
                return _leftOffset;
            }
            set
            {
                _leftOffset = value;
                UpdatePosition();
            }
        }

        public double TopOffset
        {
            get
            {
                return _topOffset;
            }
            set
            {
                _topOffset = value;
                UpdatePosition();

            }
        }

        private void UpdatePosition()
        {
            AdornerLayer adornerLayer = this.Parent as AdornerLayer;
            if (adornerLayer != null)
            {
                adornerLayer.Update(AdornedElement);
            }
        }

        public override GeneralTransform GetDesiredTransform(GeneralTransform transform)
        {
            GeneralTransformGroup result = new GeneralTransformGroup();
            result.Children.Add(base.GetDesiredTransform(transform));
            result.Children.Add(new TranslateTransform(_leftOffset, _topOffset));
            return result;
        }

        private Rectangle _child = null;
        private double _leftOffset = 0;
        private double _topOffset = 0;
    }

}